[INFRA] Import NVIDIA/CCCL upstream as optimization reference library
CCCL (CUDA C++ Core Libraries) provides: - CUB: device/block/warp-level GPU primitives (reduce, scan, sort, topk) - Thrust: high-level parallel algorithms (transform_reduce, sort, scan) - libcudacxx: CUDA C++ standard library (atomics, barriers, memory) - cudax: experimental features (memory resources, allocators) - Tuning policies: per-SM hardware-specific algorithm parameters Competition optimization vectors mapped to CCCL: - Output TPS (83% weight): warp_reduce, block_reduce, device_topk - Input TPS (14% weight): device_scan, block_load, prefetch - Cache TPS (3% weight): prefix caching strategy patterns - Memory (0.9 util): pooled/cached/buddy allocators Source: https://github.com/NVIDIA/cccl (shallow clone, HEAD only) License: Apache-2.0
This commit is contained in:
51
cccl_upstream/cudax/test/stf/reductions/many_inc.cu
Normal file
51
cccl_upstream/cudax/test/stf/reductions/many_inc.cu
Normal file
@@ -0,0 +1,51 @@
|
||||
//===----------------------------------------------------------------------===//
|
||||
//
|
||||
// Part of CUDASTF in CUDA C++ Core Libraries,
|
||||
// under the Apache License v2.0 with LLVM Exceptions.
|
||||
// See https://llvm.org/LICENSE.txt for license information.
|
||||
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
|
||||
// SPDX-FileCopyrightText: Copyright (c) 2022-2024 NVIDIA CORPORATION & AFFILIATES.
|
||||
//
|
||||
//===----------------------------------------------------------------------===//
|
||||
|
||||
#include <cuda/experimental/__stf/stream/interfaces/slice_reduction_ops.cuh>
|
||||
#include <cuda/experimental/__stf/stream/stream_ctx.cuh>
|
||||
|
||||
using namespace cuda::experimental::stf;
|
||||
|
||||
__global__ void add(int* ptr)
|
||||
{
|
||||
*ptr = *ptr + 1;
|
||||
}
|
||||
|
||||
int main()
|
||||
{
|
||||
stream_ctx ctx;
|
||||
|
||||
auto redux_op = std::make_shared<slice_reduction_op_sum<int>>();
|
||||
|
||||
int a = 17;
|
||||
|
||||
auto handle = ctx.logical_data(make_slice(&a, 1));
|
||||
|
||||
int ndevs;
|
||||
cuda_safe_call(cudaGetDeviceCount(&ndevs));
|
||||
|
||||
int K = 1024;
|
||||
for (int i = 0; i < K; i++)
|
||||
{
|
||||
// Increment the variable by 1
|
||||
ctx.task(exec_place::device(i % ndevs), handle.relaxed(redux_op))->*[](auto stream, auto s) {
|
||||
add<<<1, 1, 0, stream>>>(s.data_handle());
|
||||
};
|
||||
}
|
||||
|
||||
// Total value should be initial value + K
|
||||
ctx.task(exec_place::host(), handle.read())->*[&](auto stream, auto s) {
|
||||
cuda_safe_call(cudaStreamSynchronize(stream));
|
||||
EXPECT(s(0) == 17 + K);
|
||||
// printf("VALUE %d expected %d\n", s(0), 17 + K);
|
||||
};
|
||||
|
||||
ctx.finalize();
|
||||
}
|
||||
Reference in New Issue
Block a user